@@ -30,7 +30,9 @@ from monk import validation
 
 
 def make_dot_expanded(data):
-    if isinstance(data, dict):
+    if isinstance(data, DotExpandedDictMixin):
+        return data
+    elif isinstance(data, dict):
         pairs = []
         for key, value in data.iteritems():
             pairs.append((key, make_dot_expanded(value)))
@@ -126,12 +128,23 @@ class MongoBoundDictMixin(object):
         data = db[cls.collection].find_one(*args, **kwargs)
         if data:
             return cls.wrap_incoming(data, db)
+        else:
+            return None
 
     def save(self, db):
         assert self.collection
+
         # XXX self.structure belongs to StructuredDictMixin !!
         outgoing = dict(dict_to_db(self, self.structure))
-        db[self.collection].save(outgoing)
+
+        object_id = db[self.collection].save(outgoing)
+
+        if self.get('_id') is None:
+            self['_id'] = object_id
+        else:
+            pass
+
+        return object_id
 
 
 class StructuredDictMixin(object):
@@ -174,7 +187,7 @@ class Document(
 
     def save(self, db):
         self.validate()
-        super(Document, self).save(db)
+        return super(Document, self).save(db)
 
 
 def _db_to_dict_pairs(spec, data, db):
@@ -182,10 +195,9 @@ def _db_to_dict_pairs(spec, data, db):
         if isinstance(value, dict):
             yield key, dict(_db_to_dict_pairs(spec.get(key, {}), value, db))
         elif isinstance(value, dbref.DBRef):
-            yield key, dict(
-                db.dereference(value),
-                _id = value['_id']
-            )
+            obj = db.dereference(value)
+            cls = spec.get(key, dict)
+            yield key, cls(obj, _id=obj['_id']) if obj else None
         else:
             yield key, value
 
@@ -196,6 +208,9 @@ def dict_from_db(spec, data, db):
 
 def _dict_to_db_pairs(spec, data):
     for key, value in data.iteritems():
+        if key == '_id' and value is None:
+            # let the database assign an identifier
+            continue
         if isinstance(value, dict):
             if '_id' in value:
                 collection = spec[key].collection
